@@ -28,9 +28,10 @@
 
 #include "mos_os.h"
 #include "mos_util_debug.h"
+#include "codec_def_common.h"
 
 class CodechalDebugInterface;
-class CodechalHwInterface;
+class CodechalHwInterfaceNext;
 class CodechalSetting;
 
 //------------------------------------------------------------------------------
@@ -67,6 +68,145 @@ class CodechalSetting;
 #define CODECHAL_PUBLIC_CHK_NULL_NO_STATUS_RETURN(_ptr)                                 \
     MOS_CHK_NULL_NO_STATUS_RETURN(MOS_COMPONENT_CODEC, MOS_CODEC_SUBCOMP_PUBLIC, _ptr)
 
+#define CODECHAL_PUBLIC_CHK_STATUS_WITH_DESTROY_RETURN(_stmt, destroyFunction)                                                                                    \
+    {                                                                                                                                                             \
+        MOS_STATUS sts = (MOS_STATUS)(_stmt);                                                                                                                     \
+        MOS_CHK_COND_WITH_DESTROY_RETURN_VALUE(MOS_COMPONENT_CODEC, MOS_CODEC_SUBCOMP_ENCODE, (MOS_STATUS_SUCCESS != sts), destroyFunction, sts, "error status!") \
+    }
+
+typedef struct CODECHAL_SSEU_SETTING
+{
+    uint8_t ui8NumSlices;
+    uint8_t ui8NumSubSlices;
+    uint8_t ui8NumEUs;
+    uint8_t reserved;  // Place holder for frequency setting
+} CODECHAL_SSEU_SETTING, *PCODECHAL_SSEU_SETTING;
+
+//!
+//! \class CodechalResLock
+//! \brief Help function to lock the resource, the resource will be unlock automatically when this class destroy.
+//!
+class CodechalResLock
+{
+public:
+    //! \brief lock flag
+    enum LockFlag
+    {
+        readOnly     = 1,       //!< lock with read only
+        writeOnly    = 1 << 1,  //!< lock with write only
+        tiledAsTiled = 1 << 2,  //!< lock tile surface as tile format
+        noOverWrite  = 1 << 3,  //!< lock with no overwrite
+        noDecompress = 1 << 4,  //!< lock with no decompress
+        uncached     = 1 << 5,  //!< lock with uncached
+        forceCached  = 1 << 6   //!< lock with cache by force
+    };
+
+    //!
+    //! \brief    Constructor
+    //!
+    CodechalResLock(PMOS_INTERFACE osInterface, PMOS_RESOURCE resource) : m_os(osInterface),
+                                                                          m_res(resource),
+                                                                          m_lockedPtr(nullptr)
+    {
+    }
+
+    //!
+    //! \brief    Destructor
+    //!
+    ~CodechalResLock()
+    {
+        if (m_os != nullptr && m_res != nullptr && m_lockedPtr != nullptr)
+        {
+            m_os->pfnUnlockResource(m_os, m_res);
+        }
+    }
+
+    //!
+    //! \brief    Lock the resource
+    //! \details  Lock the resource, will be unlock automatically when this class destroy
+    //! \param  [in] lockFlags
+    //!         Combined lock flags, reference to enum LockFlag.
+    //!
+    //! \return   void pointer
+    //!           locked pointer if success, else return null pointer
+    //!
+    void *Lock(uint32_t lockFlags)
+    {
+        if (m_os == nullptr || m_res == nullptr)
+        {
+            return nullptr;
+        }
+
+        if (m_lockedPtr == nullptr)
+        {
+            MOS_LOCK_PARAMS mosLockFlags;
+            MOS_ZeroMemory(&mosLockFlags, sizeof(MOS_LOCK_PARAMS));
+
+            if ((lockFlags & readOnly) == readOnly)
+            {
+                mosLockFlags.ReadOnly = 1;
+            }
+
+            if ((lockFlags & writeOnly) == writeOnly)
+            {
+                mosLockFlags.WriteOnly = 1;
+            }
+
+            if ((lockFlags & tiledAsTiled) == tiledAsTiled)
+            {
+                mosLockFlags.TiledAsTiled = 1;
+            }
+
+            if ((lockFlags & noOverWrite) == noOverWrite)
+            {
+                mosLockFlags.NoOverWrite = 1;
+            }
+
+            if ((lockFlags & noDecompress) == noDecompress)
+            {
+                mosLockFlags.NoDecompress = 1;
+            }
+
+            if ((lockFlags & uncached) == uncached)
+            {
+                mosLockFlags.Uncached = 1;
+            }
+
+            if ((lockFlags & forceCached) == forceCached)
+            {
+                mosLockFlags.ForceCached = 1;
+            }
+
+            m_lockedPtr = m_os->pfnLockResource(
+                m_os,
+                m_res,
+                &mosLockFlags);
+        }
+
+        return m_lockedPtr;
+    }
+
+private:
+    PMOS_INTERFACE m_os;
+    PMOS_RESOURCE  m_res;
+    void          *m_lockedPtr;
+
+MEDIA_CLASS_DEFINE_END(CodechalResLock)
+};
+
+/*! \brief Settings used to create the CodecHal device.
+*/
+typedef struct _CODECHAL_STANDARD_INFO
+{
+    CODECHAL_FUNCTION CodecFunction;  //!< High level codec functionality requested.
+    uint32_t          Mode;           //!< Mode requested (high level combination between Standard and CodecFunction).
+    /*! \brief Applies to decode only, hybrid decoding requested.
+    *
+    *   Hybrid decoding uses EU kernels when FF HW is not available.
+    */
+    bool bIsHybridCodec;
+} CODECHAL_STANDARD_INFO, *PCODECHAL_STANDARD_INFO;
+
 //!
 //! \class Codechal
 //! \brief This class defines the common member fields, functions etc as Codechal base class.
@@ -82,7 +222,7 @@ public:
     //!           Debug interface
     //!
     Codechal(
-        CodechalHwInterface    *hwInterface,
+        CodechalHwInterfaceNext *hwInterface,
         CodechalDebugInterface *debugInterface);
 
     //!
@@ -130,6 +270,8 @@ public:
     //!
     virtual MOS_STATUS EndFrame();
 
+    virtual MOS_STATUS Reformat();
+
     //!
     //! \brief    Performs the operation requested by the codec function.
     //! \param    [in] params
@@ -198,7 +340,7 @@ public:
     //! \return   CodechalHwInterface
     //!           return hardware interface
     //!
-    CodechalHwInterface *GetHwInterface() { return m_hwInterface; }
+    CodechalHwInterfaceNext *GetHwInterface() { return m_hwInterface; }
 
     //!
     //! \brief    Gets OS interface.
@@ -224,7 +366,7 @@ protected:
     //! \brief    HW Inteface
     //! \details  Responsible for constructing all defined states and commands. 
     //!           Each HAL has a separate OS interface.
-    CodechalHwInterface     *m_hwInterface      = nullptr;
+    CodechalHwInterfaceNext *m_hwInterface = nullptr;
 
     //! \brief    Os Inteface
     //! \details  Used to abstract all OS and KMD interactions such that CodecHal may be 
@@ -245,6 +387,8 @@ protected:
     //! \brief    Apogeios Enable Flag
     bool                    m_apogeiosEnable = false;
 
+    //!< usersettingInstance
+    MediaUserSettingSharedPtr   m_userSettingPtr = nullptr;
 MEDIA_CLASS_DEFINE_END(Codechal)
 };
 
