@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2018-2022, Intel Corporation
+* Copyright (c) 2018-2023, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -61,16 +61,11 @@ DecodePipeline::DecodePipeline(
     {
         MOS_OS_ASSERTMESSAGE("Initialize CodechalOcaDumper failed!");
     }
-
-    CODECHAL_DEBUG_TOOL(
-        DECODE_ASSERT(debugInterface != nullptr);
-        m_debugInterface = debugInterface;
-    );
 }
 
 MOS_STATUS DecodePipeline::CreateStatusReport()
 {
-    m_statusReport = MOS_New(DecodeStatusReport, m_allocator, true);
+    m_statusReport = MOS_New(DecodeStatusReport, m_allocator, true, m_osInterface);
     DECODE_CHK_NULL(m_statusReport);
     DECODE_CHK_STATUS(m_statusReport->Create());
 
@@ -155,7 +150,6 @@ MOS_STATUS DecodePipeline::Initialize(void *settings)
     DECODE_CHK_STATUS(MediaPipeline::InitPlatform());
     DECODE_CHK_STATUS(MediaPipeline::CreateMediaCopyWrapper());
     DECODE_CHK_NULL(m_mediaCopyWrapper);
-    m_mediaCopyWrapper->CreateMediaCopyState();
 
     DECODE_CHK_NULL(m_waTable);
 
@@ -168,10 +162,12 @@ MOS_STATUS DecodePipeline::Initialize(void *settings)
         m_mediaCopyWrapper->SetMediaCopyState(m_hwInterface->CreateMediaCopy(m_osInterface));
     }
 
-#if USE_CODECHAL_DEBUG_TOOL
-    DECODE_CHK_NULL(m_debugInterface);
-    DECODE_CHK_STATUS(m_debugInterface->SetFastDumpConfig(m_mediaCopyWrapper->GetMediaCopyState()));
-#endif
+    CODECHAL_DEBUG_TOOL(
+        m_debugInterface = MOS_New(CodechalDebugInterface);
+        DECODE_CHK_NULL(m_debugInterface);
+        DECODE_CHK_STATUS(
+            m_debugInterface->Initialize(m_hwInterface, codecSettings->codecFunction, m_mediaCopyWrapper));
+    );
 
     m_mediaContext = MOS_New(MediaContext, scalabilityDecoder, m_hwInterface, m_osInterface);
     DECODE_CHK_NULL(m_mediaContext);
@@ -190,7 +186,7 @@ MOS_STATUS DecodePipeline::Initialize(void *settings)
     m_decodecp = Create_DecodeCpInterface(codecSettings, m_hwInterface->GetCpInterface(), m_hwInterface->GetOsInterface());
     if (m_decodecp)
     {
-        m_decodecp->RegisterParams(codecSettings);
+        DECODE_CHK_STATUS(m_decodecp->RegisterParams(codecSettings));
     }
     DECODE_CHK_STATUS(CreateFeatureManager());
     DECODE_CHK_STATUS(m_featureManager->Init(codecSettings));
@@ -198,6 +194,14 @@ MOS_STATUS DecodePipeline::Initialize(void *settings)
     DECODE_CHK_STATUS(CreateSubPipeLineManager(codecSettings));
     DECODE_CHK_STATUS(CreateSubPacketManager(codecSettings));
 
+#if (_DEBUG || _RELEASE_INTERNAL)
+    m_delayMiliseconds = ReadUserFeature(m_userSettingPtr, "Delay Miliseconds", MediaUserSetting::Group::Sequence).Get<uint32_t>();
+#endif
+
+#if MHW_HWCMDPARSER_ENABLED
+    mhw::HwcmdParser::InitInstance(m_osInterface, mhw::HwcmdParser::AddOnMode::NoAddOn);
+#endif
+
     return MOS_STATUS_SUCCESS;
 }
 
@@ -223,6 +227,8 @@ MOS_STATUS DecodePipeline::Uninitialize()
 
     MOS_Delete(m_allocator);
 
+    CODECHAL_DEBUG_TOOL(MOS_Delete(m_debugInterface););
+
     return MOS_STATUS_SUCCESS;
 }
 
@@ -252,11 +258,16 @@ uint8_t DecodePipeline::GetSystemVdboxNumber()
 {
     uint8_t numVdbox = 1;
 
-    MEDIA_SYSTEM_INFO *gtSystemInfo = m_osInterface->pfnGetGtSystemInfo(m_osInterface);
-    if (gtSystemInfo != nullptr)
+    MEDIA_ENGINE_INFO mediaSysInfo;
+    MOS_ZeroMemory(&mediaSysInfo, sizeof(MEDIA_ENGINE_INFO));
+    MOS_STATUS eStatus = m_osInterface->pfnGetMediaEngineInfo(m_osInterface, mediaSysInfo);
+    if (eStatus == MOS_STATUS_SUCCESS)
     {
-        // Both VE mode and media solo mode should be able to get the VDBOX number via the same interface
-        numVdbox = (uint8_t)(gtSystemInfo->VDBoxInfo.NumberOfVDBoxEnabled);
+        numVdbox = (uint8_t)(mediaSysInfo.VDBoxInfo.NumberOfVDBoxEnabled);
+    }
+    else
+    {
+        DECODE_ASSERTMESSAGE("Failed to query media engine info!!");
     }
 
     return numVdbox;
@@ -279,12 +290,21 @@ MOS_STATUS DecodePipeline::Prepare(void *params)
     DECODE_CHK_STATUS(m_featureManager->Update(decodeParams));
     if (m_decodecp)
     {
-        m_decodecp->UpdateParams(true);
+        DECODE_CHK_STATUS(m_decodecp->UpdateParams(true));
     }
     DECODE_CHK_STATUS(m_subPacketManager->Prepare());
 
     DECODE_CHK_STATUS(Mos_Solo_SetGpuAppTaskEvent(m_osInterface, decodeParams->m_gpuAppTaskEvent));
 
+#if MHW_HWCMDPARSER_ENABLED
+    auto instance = mhw::HwcmdParser::GetInstance();
+    if (instance)
+    {
+        // use I as default frame type
+        instance->Update('I', (void *)m_featureManager, mhw::HwcmdParser::WorkloadType::Decode);
+    }
+#endif
+
     return MOS_STATUS_SUCCESS;
 }
 
@@ -407,11 +427,78 @@ MOS_STATUS DecodePipeline::DumpDownSamplingParams(DecodeDownSamplingFeature &dow
 }
 #endif
 
+MOS_STATUS DecodePipeline::DumpBitstream(PMOS_RESOURCE pBitstream, uint32_t size, uint32_t offset, const char* attrName)
+{
+    DECODE_FUNC_CALL();
+
+    DECODE_CHK_NULL(pBitstream);
+
+    if(attrName == nullptr)
+    {
+        attrName = CodechalDbgAttr::attrDecodeBitstream;
+    }
+
+    DECODE_CHK_STATUS(m_debugInterface->DumpBuffer(
+        pBitstream, 
+        attrName,
+        "_DEC", 
+        size, 
+        offset, 
+        CODECHAL_NUM_MEDIA_STATES));
+
+    if (MOS_TraceKeyEnabled(TR_KEY_DECODE_BITSTREAM_INFO))
+    {
+        MOS_LOCK_PARAMS lockFlags;
+        MOS_ZeroMemory(&lockFlags, sizeof(MOS_LOCK_PARAMS));
+        lockFlags.ReadOnly = 1;
+        
+        uint8_t *data = (uint8_t *)m_osInterface->pfnLockResource(m_osInterface, pBitstream, &lockFlags);
+        MEDIA_DEBUG_CHK_NULL(data);
+        data += offset;
+
+        DECODE_EVENTDATA_BITSTREAM eventData;
+        eventData.BitstreamSize = size;
+        for (uint32_t i = 0; i < MOS_MIN(BITSTREAM_INFO_SIZE, size); i++)
+        {
+            eventData.Data[i] = data[i];
+        }
+        MOS_TraceEvent(EVENT_DECODE_INFO_BITSTREAM, EVENT_TYPE_INFO, &eventData, sizeof(eventData));
+
+        m_osInterface->pfnUnlockResource(m_osInterface, pBitstream);
+    }
+
+    return MOS_STATUS_SUCCESS;
+}
+
+MOS_STATUS DecodePipeline::DelayForDumpOutput()
+{
+    DECODE_FUNC_CALL();
+
+    if (m_debugInterface->DumpIsEnabled(CodechalDbgAttr::attrDelayForDumpOutput))
+    {
+        uint32_t completedCount = m_statusReport->GetCompletedCount();
+        for (uint8_t i = 0; i < 20; i++)
+        {
+            if (completedCount <= m_statusCheckCount)
+            {
+                MosUtilities::MosSleep(5);
+                completedCount = m_statusReport->GetCompletedCount();
+            }
+            else
+            {
+                break;
+            }
+        }
+    }
+
+    return MOS_STATUS_SUCCESS;
+}
+
 MOS_STATUS DecodePipeline::DumpOutput(const DecodeStatusReportData& reportData)
 {
     DECODE_FUNC_CALL();
 
-    if (m_debugInterface->DumpIsEnabled(CodechalDbgAttr::attrDecodeOutputSurface) || MOS_TraceKeyEnabled(TR_KEY_DECODE_DSTYUV))
+    if (m_debugInterface->DumpIsEnabled(CodechalDbgAttr::attrDecodeOutputSurface))
     {
         MOS_SURFACE dstSurface;
         MOS_ZeroMemory(&dstSurface, sizeof(dstSurface));
@@ -423,23 +510,25 @@ MOS_STATUS DecodePipeline::DumpOutput(const DecodeStatusReportData& reportData)
             &dstSurface, CodechalDbgAttr::attrDecodeOutputSurface, "DstSurf"));
     }
 
+    if(m_streamout != nullptr)
+    {
+        m_streamout->DumpOutput(reportData);
+    }
+
 #ifdef _DECODE_PROCESSING_SUPPORTED
     DecodeDownSamplingFeature* downSamplingFeature = dynamic_cast<DecodeDownSamplingFeature*>(
         m_featureManager->GetFeature(DecodeFeatureIDs::decodeDownSampling));
     if (downSamplingFeature != nullptr && downSamplingFeature->IsEnabled())
     {
-        if (reportData.currSfcOutputPicRes != nullptr &&
+        if (reportData.currSfcOutputSurface != nullptr &&
             m_debugInterface->DumpIsEnabled(CodechalDbgAttr::attrSfcOutputSurface))
         {
-            MOS_SURFACE sfcDstSurface;
-            MOS_ZeroMemory(&sfcDstSurface, sizeof(sfcDstSurface));
-            sfcDstSurface.Format     = Format_NV12;
-            sfcDstSurface.OsResource = *reportData.currSfcOutputPicRes;
+            MOS_SURFACE &sfcDstSurface = *reportData.currSfcOutputSurface;
 
             if (!Mos_ResourceIsNull(&sfcDstSurface.OsResource))
             {
-#if (_DEBUG || _RELEASE_INTERNAL)
                 DECODE_CHK_STATUS(m_allocator->GetSurfaceInfo(&sfcDstSurface));
+
                 //rgb format read from reg key
                 uint32_t sfcOutputRgbFormatFlag =
                     ReadUserFeature(m_userSettingPtr, "Decode SFC RGB Format Output", MediaUserSetting::Group::Sequence).Get<uint32_t>();
@@ -453,7 +542,6 @@ MOS_STATUS DecodePipeline::DumpOutput(const DecodeStatusReportData& reportData)
                     DECODE_CHK_STATUS(m_debugInterface->DumpYUVSurface(
                         &sfcDstSurface, CodechalDbgAttr::attrSfcOutputSurface, "SfcDstSurf"));
                 }
-#endif
             }
         }
 
@@ -473,112 +561,6 @@ MOS_STATUS DecodePipeline::DumpOutput(const DecodeStatusReportData& reportData)
 }
 #endif
 
-#if MOS_EVENT_TRACE_DUMP_SUPPORTED
-MOS_STATUS DecodePipeline::TraceDataDumpOutput(const DecodeStatusReportData &reportData)
-{
-    bool bAllocate = false;
-    MOS_SURFACE dstSurface;
-    MOS_ZeroMemory(&dstSurface, sizeof(dstSurface));
-    dstSurface.OsResource = reportData.currDecodedPicRes;
-    DECODE_CHK_STATUS(m_allocator->GetSurfaceInfo(&dstSurface));
-
-    if (!m_allocator->ResourceIsNull(&dstSurface.OsResource))
-    {
-        if (m_tempOutputSurf == nullptr || m_allocator->ResourceIsNull(&m_tempOutputSurf->OsResource))
-        {
-            bAllocate = true;
-        }
-        else if (m_tempOutputSurf->dwWidth  < dstSurface.dwWidth ||
-                 m_tempOutputSurf->dwHeight < dstSurface.dwHeight)
-        {
-            bAllocate = true;
-        }
-        else
-        {
-            bAllocate = false;
-        }
-
-        if (bAllocate)
-        {
-            if (!m_allocator->ResourceIsNull(&m_tempOutputSurf->OsResource))
-            {
-                m_allocator->Destroy(m_tempOutputSurf);
-            }
-
-            m_tempOutputSurf = m_allocator->AllocateLinearSurface(
-                dstSurface.dwWidth,
-                dstSurface.dwHeight,
-                "Decode Output Surf",
-                dstSurface.Format,
-                dstSurface.bIsCompressed,
-                resourceOutputPicture,
-                lockableSystemMem,
-                MOS_TILE_LINEAR_GMM);
-        }
-
-        DECODE_CHK_STATUS(m_osInterface->pfnDoubleBufferCopyResource(
-            m_osInterface,
-            &dstSurface.OsResource,
-            &m_tempOutputSurf->OsResource,
-            false));
-
-        DECODE_EVENTDATA_YUV_SURFACE_INFO eventData =
-        {
-            (uint32_t)reportData.currDecodedPic.PicFlags,
-            reportData.frameType,
-            m_tempOutputSurf->dwOffset,
-            m_tempOutputSurf->YPlaneOffset.iYOffset,
-            m_tempOutputSurf->dwPitch,
-            m_tempOutputSurf->dwWidth,
-            m_tempOutputSurf->dwHeight,
-            (uint32_t)m_tempOutputSurf->Format,
-            m_tempOutputSurf->UPlaneOffset.iLockSurfaceOffset,
-            m_tempOutputSurf->VPlaneOffset.iLockSurfaceOffset,
-            m_tempOutputSurf->UPlaneOffset.iSurfaceOffset,
-            m_tempOutputSurf->VPlaneOffset.iSurfaceOffset,
-        };
-        MOS_TraceEvent(EVENT_DECODE_DUMPINFO_DST, EVENT_TYPE_INFO, &eventData, sizeof(eventData), NULL, 0); 
-
-        ResourceAutoLock resLock(m_allocator, &m_tempOutputSurf->OsResource);
-        auto             pData = (uint8_t *)resLock.LockResourceForRead();
-        DECODE_CHK_NULL(pData);
-
-        MOS_TraceDataDump(
-            "Decode_OutputSurf",
-            0,
-            pData,
-            (uint32_t)m_tempOutputSurf->OsResource.pGmmResInfo->GetSizeMainSurface());
-        
-        m_allocator->UnLock(&m_tempOutputSurf->OsResource);
-    }
-
-    return MOS_STATUS_SUCCESS;
-}
-
-MOS_STATUS DecodePipeline::TraceDataDump2ndLevelBB(PMHW_BATCH_BUFFER batchBuffer)
-{
-    DECODE_FUNC_CALL();
-
-    DECODE_CHK_NULL(batchBuffer);
-    batchBuffer->iLastCurrent = batchBuffer->iSize * batchBuffer->count;
-    batchBuffer->dwOffset     = 0;
-
-    ResourceAutoLock resLock(m_allocator, &batchBuffer->OsResource);
-    auto             pData = (uint8_t *)resLock.LockResourceForRead();
-    DECODE_CHK_NULL(pData);
-
-    MOS_TraceDataDump(
-        "Decode_2ndLevelCmdBB",
-        m_osInterface->pfnGetGpuContext(m_osInterface),
-        pData,
-        batchBuffer->iLastCurrent);
-
-    m_allocator->UnLock(&batchBuffer->OsResource);
-
-    return MOS_STATUS_SUCCESS;
-}
-#endif
-
 #if (_DEBUG || _RELEASE_INTERNAL)
 MOS_STATUS DecodePipeline::ReportVdboxIds(const DecodeStatusMfx& status)
 {
@@ -615,15 +597,11 @@ MOS_STATUS DecodePipeline::ReportSfcLinearSurfaceUsage(const DecodeStatusReportD
         m_featureManager->GetFeature(DecodeFeatureIDs::decodeDownSampling));
     if (downSamplingFeature != nullptr && downSamplingFeature->IsEnabled())
     {
-        if (reportData.currSfcOutputPicRes != nullptr)
+        if (reportData.currSfcOutputSurface != nullptr)
         {
-            MOS_SURFACE sfcDstSurface;
-            MOS_ZeroMemory(&sfcDstSurface, sizeof(sfcDstSurface));
-            sfcDstSurface.Format     = Format_NV12;
-            sfcDstSurface.OsResource = *reportData.currSfcOutputPicRes;
+            MOS_SURFACE sfcDstSurface = *reportData.currSfcOutputSurface;
             if (!Mos_ResourceIsNull(&sfcDstSurface.OsResource))
             {
-                DECODE_CHK_STATUS(m_allocator->GetSurfaceInfo(&sfcDstSurface));
                 if (sfcDstSurface.TileType == MOS_TILE_LINEAR)
                 {
                     WriteUserFeature(__MEDIA_USER_FEATURE_VALUE_SFC_LINEAR_OUTPUT_USED_ID, 1, m_osInterface->pOsContext);
@@ -643,6 +621,15 @@ MOS_STATUS DecodePipeline::StatusCheck()
 {
     DECODE_FUNC_CALL();
 
+#if (_DEBUG || _RELEASE_INTERNAL)
+    if (m_delayMiliseconds > 0)
+    {
+        MosUtilities::MosSleep(m_delayMiliseconds);
+    }
+#endif
+
+    CODECHAL_DEBUG_TOOL(DECODE_CHK_STATUS(DelayForDumpOutput()));
+
     uint32_t completedCount = m_statusReport->GetCompletedCount();
     if (completedCount <= m_statusCheckCount)
     {
@@ -675,26 +662,30 @@ MOS_STATUS DecodePipeline::StatusCheck()
         auto bufferDumpNumTemp = m_debugInterface->m_bufferDumpFrameNum;
         auto currPicTemp       = m_debugInterface->m_currPic;
         auto frameTypeTemp     = m_debugInterface->m_frameType;
+        auto secondField       = m_debugInterface->m_secondField;
 
-        m_debugInterface->m_bufferDumpFrameNum = m_statusCheckCount;
+        m_debugInterface->m_bufferDumpFrameNum = DecodeOutputIndex;
         m_debugInterface->m_currPic            = reportData.currDecodedPic;
         m_debugInterface->m_frameType          = reportData.frameType;
+        m_debugInterface->m_secondField        = reportData.secondField;
+
 #ifdef _DECODE_PROCESSING_SUPPORTED
         ReportSfcLinearSurfaceUsage(reportData);
 #endif
+
         DECODE_CHK_STATUS(DumpOutput(reportData));
 
+        if ((CodecHal_PictureIsFrame(m_debugInterface->m_currPic)) ||
+            (CodecHal_PictureIsField(m_debugInterface->m_currPic) && m_debugInterface->m_secondField) ||
+            (!CodecHal_PictureIsFrame(m_debugInterface->m_currPic) && !CodecHal_PictureIsField(m_debugInterface->m_currPic)))
+        {            
+            DecodeOutputIndex++;
+        }
+
         m_debugInterface->m_bufferDumpFrameNum = bufferDumpNumTemp;
         m_debugInterface->m_currPic            = currPicTemp;
         m_debugInterface->m_frameType          = frameTypeTemp;
-#endif
-
-#if MOS_EVENT_TRACE_DUMP_SUPPORTED
-        if (MOS_TraceKeyEnabled(TR_KEY_DECODE_DSTYUV_IN_TRACE))
-        {
-            const DecodeStatusReportData &reportETWData = statusReport->GetReportData(m_statusCheckCount);
-            DECODE_CHK_STATUS(TraceDataDumpOutput(reportETWData));
-        }
+        m_debugInterface->m_secondField        = secondField;
 #endif
 
         m_statusCheckCount++;
