@@ -222,6 +222,11 @@ VpResourceManager::~VpResourceManager()
         m_allocator.DestroyVpSurface(m_vebox1DLookUpTables);
     }
 
+    if (m_innerTileConvertInput)
+    {
+        m_allocator.DestroyVpSurface(m_innerTileConvertInput);
+    }
+
     if (m_temperalInput)
     {
         m_allocator.DestroyVpSurface(m_temperalInput);
@@ -246,6 +251,13 @@ VpResourceManager::~VpResourceManager()
 
     m_allocator.DestroyVpSurface(m_cmfcCoeff);
     m_allocator.DestroyVpSurface(m_decompressionSyncSurface);
+    for (int i = 0; i < VP_COMP_MAX_LAYERS; ++i)
+    {
+        if (m_fcIntermediaSurfaceInput[i])
+        {
+            m_allocator.DestroyVpSurface(m_fcIntermediaSurfaceInput[i]);
+        }
+    }
 
     m_allocator.CleanRecycler();
 }
@@ -1077,6 +1089,49 @@ MOS_STATUS VpResourceManager::AssignFcResources(VP_EXECUTE_CAPS &caps, std::vect
         allocated));
     surfSetting.surfGroup.insert(std::make_pair(SurfaceTypeDecompressionSync, m_decompressionSyncSurface));
 
+    // Allocate L0 fc intermedia Surface
+    for (uint32_t i = 0; i < inputSurfaces.size(); ++i)
+    {
+        MOS_FORMAT fcIntermediaSurfaceInputFormat = Format_Any;
+        switch (inputSurfaces[i]->osSurface->Format)
+        {
+        case Format_RGBP:
+        case Format_BGRP:
+            fcIntermediaSurfaceInputFormat = Format_A8R8G8B8;
+            break;
+        case Format_444P:
+            fcIntermediaSurfaceInputFormat = Format_AYUV;
+            break;
+        case Format_I420:
+        case Format_YV12:
+            fcIntermediaSurfaceInputFormat = Format_NV12;
+            break;
+        default:
+            break;
+        }
+        if (fcIntermediaSurfaceInputFormat != Format_Any)
+        {
+            VP_PUBLIC_CHK_STATUS_RETURN(m_allocator.ReAllocateSurface(
+                m_fcIntermediaSurfaceInput[i],
+                "fcIntermediaSurfaceInput",
+                fcIntermediaSurfaceInputFormat,
+                MOS_GFXRES_2D,
+                MOS_TILE_Y,
+                inputSurfaces[i]->osSurface->dwWidth,
+                inputSurfaces[i]->osSurface->dwHeight,
+                false,
+                MOS_MMC_DISABLED,
+                allocated,
+                false,
+                IsDeferredResourceDestroyNeeded(),
+                MOS_HW_RESOURCE_USAGE_VP_INTERNAL_READ_WRITE_RENDER));
+
+            m_fcIntermediaSurfaceInput[i]->rcSrc      = inputSurfaces[i]->rcSrc;
+            m_fcIntermediaSurfaceInput[i]->rcDst      = inputSurfaces[i]->rcDst;
+            m_fcIntermediaSurfaceInput[i]->SampleType = inputSurfaces[i]->SampleType;
+            surfSetting.surfGroup.insert(std::make_pair((SurfaceType)(SurfaceTypeFcIntermediaInput + i), m_fcIntermediaSurfaceInput[i]));
+        }
+    }
     return MOS_STATUS_SUCCESS;
 }
 
@@ -1888,7 +1943,27 @@ MOS_STATUS VpResourceManager::AllocateVeboxResource(VP_EXECUTE_CAPS& caps, VP_SU
         }
     }
     // cappipe
+    if (caps.enableSFCLinearOutputByTileConvert)
+    {
+        VP_PUBLIC_CHK_STATUS_RETURN(m_allocator.ReAllocateSurface(
+            m_innerTileConvertInput,
+            "TempTargetSurface",
+            outputSurface->osSurface->Format,
+            MOS_GFXRES_2D,
+            MOS_TILE_Y,
+            outputSurface->osSurface->dwWidth,
+            outputSurface->osSurface->dwHeight,
+            false,
+            MOS_MMC_DISABLED,
+            bAllocated,
+            false,
+            IsDeferredResourceDestroyNeeded()));
 
+        m_innerTileConvertInput->ColorSpace = outputSurface->ColorSpace;
+        m_innerTileConvertInput->rcSrc      = outputSurface->rcSrc;
+        m_innerTileConvertInput->rcDst      = outputSurface->rcDst;
+        m_innerTileConvertInput->rcMaxSrc   = outputSurface->rcMaxSrc;
+    }
     return MOS_STATUS_SUCCESS;
 }
 
@@ -2233,6 +2308,11 @@ MOS_STATUS VpResourceManager::AssignVeboxResource(VP_EXECUTE_CAPS& caps, VP_SURF
         surfGroup.insert(std::make_pair(SurfaceType1k1dLut, m_vebox1DLookUpTables));
     }
 
+    if (caps.enableSFCLinearOutputByTileConvert)
+    {
+        surfGroup.insert(std::make_pair(SurfaceTypeInnerTileConvertInput, m_innerTileConvertInput));
+    }
+
     // Update previous Dn output flag for next frame to use.
     if (surfGroup.find(SurfaceTypeDNOutput) != surfGroup.end() || m_sameSamples && m_pastDnOutputValid)
     {
