@@ -552,7 +552,8 @@ protected:
 
     // Fc Resource
     VP_SURFACE *m_cmfcCoeff                                   = nullptr;
-    VP_SURFACE *m_fcIntermediaSurfaceInput[8]                 = {};
+    VP_SURFACE *m_fcIntermediaSurfaceOutput                         = nullptr;
+    VP_SURFACE *m_fcIntermediaSurfaceInput[VP_COMP_MAX_LAYERS]      = {};
     //for decompreesion sync on interlace input of FC
     VP_SURFACE *m_decompressionSyncSurface                                  = nullptr;
     // Hdr Resource
