@@ -53,6 +53,7 @@
 #include "igvpkrn_l0_xe2_hpg.h"
 #include "igvpfc_common_xe2.h"
 #include "igvpfc_fp_xe2.h"
+#include "igvpfc_420PL3_input_xe2.h"
 #include "igvpfc_444PL3_input_xe2.h"
 #endif
 
@@ -152,6 +153,7 @@ void VphalInterfacesXe2_Lpm::InitPlatformKernelBinary(
     vpPlatformInterface->AddVpNativeAdvKernelEntryToList(IGVP3DLUT_GENERATION_XE2_HPG, IGVP3DLUT_GENERATION_XE2_HPG_SIZE, "hdr_3dlut_l0");
     AddVpNativeKernelEntryToListFc_commonXe2(*vpPlatformInterface);
     AddVpNativeKernelEntryToListFc_fpXe2(*vpPlatformInterface);
+    AddVpNativeKernelEntryToListFc_420pl3_inputXe2(*vpPlatformInterface);
     AddVpNativeKernelEntryToListFc_444pl3_inputXe2(*vpPlatformInterface);
 #endif
 }
